#!/bin/sh
":"; # -*- mode: emacs-lisp; lexical-binding: t; -*-
":"; exec emacs --no-init-file --script "$0" "$@" -- END
;; "-- END" is added to allow handling of standard Emacs options
;; and to execute this script when command line options are exhausted.

(add-to-list 'load-path "~/org-mode/lisp/")
(add-to-list 'load-path "/usr/share/emacs/site-lisp/elpa-src/htmlize-1.56")
(require 'ox-html)
(require 'cl-seq)

(defvar publish-sh-maxfail 16
  "Limit number of failures to abort with non-zero exit code.")

(defun publish-sh-main ()
  (setq make-backup-files nil
	debug-on-error t)

  (setq org-confirm-babel-evaluate nil
	org-html-style-default ""
	org-html-scripts ""
	org-html-htmlize-output-type 'css
	org-html-doctype "html5"
	org-html-html5-fancy t
	org-html-validation-link nil
	org-plantuml-jar-path "/usr/share/plantuml/plantuml.jar"
	org-ditaa-jar-path "/usr/bin/ditaa"
	org-html-preamble
	(with-temp-buffer (insert-file-contents "preamble.html") (buffer-string))
	org-html-postamble
	"<div id=\"license\"><p>Documentation from the orgmode.org/worg/ website (either in its HTML format or in its Org format) is licensed under the <a href=\"http://www.gnu.org/copyleft/fdl.html\">GNU Free Documentation License version 1.3</a> or later.  The code examples and css stylesheets are licensed under the <a href=\"http://www.gnu.org/licenses/gpl.html\">GNU General Public License v3</a> or later.</p></div>"
	org-html-head
	"<link rel=\"stylesheet\" title=\"Standard\" href=\"/worg/style/worg.css\" type=\"text/css\" />
<link rel=\"alternate stylesheet\" title=\"Zenburn\" href=\"/worg/style/worg-zenburn.css\" type=\"text/css\" />
<link rel=\"alternate stylesheet\" title=\"Classic\" href=\"/worg/style/worg-classic.css\" type=\"text/css\" />
<link rel=\"icon\" href=\"/favicon.ico\" type=\"image/x-icon\" />")

  (org-babel-do-load-languages
   'org-babel-load-languages
   '((emacs-lisp . t)
     (shell . t)
     (dot . t)
     (clojure . t)
     (org . t)
     (ditaa . t)
     (org . t)
     (scheme . t)
     (plantuml . t)
     (R . t)
     (gnuplot . t)))

  (let ((fail-count 0)
	fail-files)
    (dolist (org-file (cl-remove-if
		       (lambda (n) (string-match-p "worg/archive/" n))
		       (directory-files-recursively default-directory "\\.org$")))
      (let ((html-file (concat (file-name-directory org-file)
			       (file-name-base org-file) ".html"))
	    (org-relative (file-relative-name org-file default-directory)))
	(if (and (file-exists-p html-file)
		 (file-newer-than-file-p html-file org-file))
	    (message " [skipping] unchanged %s" org-file)
	  (message "[exporting] %s" org-relative)
	  (with-current-buffer (find-file org-file)
	    (condition-case err
		(org-html-export-to-html)
	      (error
	       (let ((error-text (error-message-string err)))
		 (message "%s" error-text)
		 (setq fail-count (1+ fail-count))
		 (when (> publish-sh-maxfail 0)
		   (push (format "%s: %s" org-relative error-text) fail-files)
		   (unless (< fail-count publish-sh-maxfail)
		     (terpri)
		     (dolist (e fail-files)
		       (message "%s" e))
		     (message "%d files failed" fail-count)
		     (signal (car err) (cdr err)))))))))))
    (dolist (e fail-files)
      (message "%s" e))))

;;; Command line options

(defvar publish-sh-switch-alist
  `(("--maxfail" . ,(lambda (val) (setq publish-sh-maxfail 
					(string-to-number val)))))
  "Script command line options.

When the code is executed, it is too late to `append' options
to `command-switch-alist', so a custom parser is necessary.")

(defun publish-sh-handle-command-line-option (options-alist)
  "A helper to define an element of `command-line-functions'.

Return `t' if `argi' contains an option from OPTIONS-ALIST
and call the function from the same (\"--option\" . func) cons.
The handler is called with an argument that is option value
\"--option=VAL\" or \"--option VAL\". In the latter case
the value is taken from the next element of `command-line-args-left'."
  (let* ((option
	  (and (string-match "^\\(--[^=-][^=]*\\)\\(?:=\\(.*\\)\\)?$" argi)
	       (match-string 1 argi)))
	 (completion (and option (try-completion option options-alist)))
	 (handler (and completion
		       (cdr (assoc (if (eq t completion) option completion)
				   options-alist)))))
    (cond
     (handler
      (let ((val (or (match-string 2 argi)
		     (pop command-line-args-left))))
	(unless val
	  (error "Argument of %s not specified" option))
	(funcall handler val))
      t)
     (completion (error "Option `%s' is ambiguous" completion)))))

(defun publish-sh-args-done (func)
  "A helper to call FUNC when no command arguments left.

To be called from an element of `command-line-functions'.
Add \"-- END\" at the end of the command line."
  ;; Emacs startup.el ignores "--" even after "--"
  ;; After "--" `argi' is always "", however before it
  ;; the variable may contain either file name or an option.
  ;; See `command-line-1'.
  (unless command-line-args-left  ; last "END"
    (funcall func)
    t))

(push (lambda ()
	(publish-sh-handle-command-line-option publish-sh-switch-alist))
      command-line-functions)
(push (lambda()
	(publish-sh-args-done #'publish-sh-main))
      command-line-functions)
